Attribute VB_Name = "z_UIMessages"
Option Private Module
Option Explicit

Public Enum MsgCode
  MSG_OK = 0
  
  EM_SYNC_DOC = ERR_SYNC_DOC
  EM_SYNC_DB = ERR_SYNC_DB
  EM_SYNC_LINK_ID = ERR_SYNC_LINK_ID
  EM_LINKS_MISSING = ERR_LINKS_MISSING
  EM_INVALID_LINK_NUMBER = ERR_INVALID_LINK_NUMBER
  EM_LINK_NO_FILE = ERR_LINK_NO_FILE
  EM_BOOKMARK_INVALID = ERR_BOOKMARK_INVALID
  EM_LINK_SELF = ERR_LINK_SELF
  EM_ADD_LINK_DB_FAILED = ERR_ADD_LINK_DB_FAILED
  EM_ADD_LINK_DOC_FAILED = ERR_ADD_LINK_DOC_FAILED
  
  EM_DOC_NOT_ADDED = vbObjectError + 1
  EM_DOC_FILE_EXISTS
  EM_TITLELINK_INVALID
  EM_SELECTION_INVALID
  EM_FILENAME_TAKEN
  EM_LINKS_NOT_ZERO
  EM_PARAGRAPH_MULTIPLE
  EM_LINK_COUNT
  EM_LINK_SINGLE
  EM_LINK_INVALID
  EM_DOC_NO_LINKS
  EM_HELP_FILE_INVALID
  EM_REMOVE_FIRST_LINK
  EM_NO_BOOKMARKS
  EM_FILE_ALREADY_ADDED
  EM_BOOKMARK_NOT_AVAILABLE
  EM_NO_VALID_LINKS
  EM_FILENAME_EMPTY
  EM_FILENAME_TOO_LONG
  EM_HAS_NO_LINKS
  EM_ID_INVALID_FORMAT
  EM_SELECTION_NOT_IN_MAIN
  
  IM_SCAN_MODE
  IM_IGNORE_MODE
  IM_LINKS_TODO
  IM_LINKS_DONE
  IM_DOC_OUTER
  IM_LINKS_ADDED
  IM_DOC_IGNORED
  IM_DOC_LINKS_DONE
  IM_HYPERLINKS_DONE
  IM_QUERY_LINK
  IM_DOC_EXPRESS_ADD
  IM_REMOVE_LINK_OK
  
  QM_ADD_TITLELINK
  QM_OPEN_GARANT
  QM_DELETE_BEFORE_ADD
  QM_DELETE_BEFORE_ADD_DUMMY
  QM_RELOAD_ON_UPDATE
  QM_RELOAD_CONFIRM
End Enum

Private g_UI As API_UserInteraction

Public Function UserInteraction() As API_UserInteraction
  If g_UI Is Nothing Then _
    Set g_UI = New API_UserInteraction
  Set UserInteraction = g_UI
End Function

Public Function SetUserInteraction(newUI As API_UserInteraction)
  Set g_UI = newUI
End Function

Public Function ProcessErrorMessages(expectedErrors As Scripting.Dictionary)
  If Err.Number = 0 Then _
    Exit Function
    
  Call Unload(CSE_ProgressBar)
  If Not expectedErrors.Exists(Err.Number) Then _
    Call Err.Raise(Err.Number)
  
  Select Case Err.Number
    Case ERR_LINK_NO_FILE: Call UserInteraction.ShowMessage(Err.Number, Err.Source)
    Case ERR_BOOKMARK_INVALID: Call UserInteraction.ShowMessage(Err.Number, Err.Source)
    Case ERR_INVALID_LINK_NUMBER: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_SYNC_DB: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_SYNC_DOC: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_LINK_SELF: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_LINKS_MISSING: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_ADD_LINK_DB_FAILED: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_ADD_LINK_DOC_FAILED: Call UserInteraction.ShowMessage(Err.Number)
    Case ERR_SYNC_LINK_ID: Call UserInteraction.ShowMessage(Err.Number, Err.Description)
    
    Case Else: Call UserInteraction.ShowMessage(Err.Number)
  End Select
  
  Call Err.Clear
  On Error GoTo 0
End Function

Public Function UIShowMessage(theCode As MsgCode, ParamArray params() As Variant)
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Select Case theCode
  Case EM_DOC_NOT_ADDED:        Call MsgBox("    ", vbExclamation)
  Case EM_DOC_FILE_EXISTS:      Call MsgBox(Fmt("!  ID {1}   : {2}", unwrapped), vbExclamation, " ")
  Case EM_SYNC_DOC:             Call MsgBox("!         !" & vbNewLine & "  ", vbCritical)
  Case EM_SYNC_DB:              Call MsgBox("!        !" & vbNewLine & "  ", vbCritical)
  Case EM_TITLELINK_INVALID:    Call MsgBox("        !", vbExclamation, "  ")
  Case EM_LINK_SELF:            Call MsgBox("     ", vbExclamation)
  Case EM_LINKS_MISSING:        Call MsgBox("!        .        ", vbExclamation)
  Case EM_SYNC_LINK_ID:         Call MsgBox(Fmt("!    ! {1}" & vbNewLine & "  ", unwrapped), vbCritical)
  
  Case EM_SELECTION_INVALID:    Call MsgBox("  .   ", vbExclamation)
  Case EM_SELECTION_NOT_IN_MAIN: Call MsgBox("    . ,      ", vbExclamation)
  Case EM_LINK_NO_FILE:         Call MsgBox(Fmt(",        " & vbNewLine & "ID: {1}", unwrapped), vbExclamation)
  Case EM_BOOKMARK_INVALID:     Call MsgBox(Fmt("   : {1}", unwrapped), vbExclamation)
  Case EM_FILENAME_TAKEN:       Call MsgBox("        !", vbExclamation)
  Case EM_LINKS_NOT_ZERO:       Call MsgBox("      ", vbExclamation)
  Case EM_PARAGRAPH_MULTIPLE:   Call MsgBox("      ", vbExclamation)
  Case EM_LINK_COUNT:           Call MsgBox("      ", vbExclamation)
  Case EM_LINK_SINGLE:          Call MsgBox("  ", vbExclamation)
  Case EM_LINK_INVALID:         Call MsgBox("      ", vbExclamation)
  Case EM_DOC_NO_LINKS:         Call MsgBox("    ,    ", vbExclamation)
  Case EM_HELP_FILE_INVALID:    Call MsgBox(Fmt("    : {1}", unwrapped), vbExclamation)
  Case EM_REMOVE_FIRST_LINK:    Call MsgBox("   !", vbExclamation)
  Case EM_NO_BOOKMARKS:         Call MsgBox("    ", vbExclamation)
  Case EM_FILE_ALREADY_ADDED:   Call MsgBox("    .   """"    ", vbExclamation)
  Case EM_INVALID_LINK_NUMBER:  Call MsgBox("    .      ", vbExclamation)
  Case EM_ADD_LINK_DB_FAILED:   Call MsgBox("      ", vbExclamation)
  Case EM_ADD_LINK_DOC_FAILED:  Call MsgBox("      ", vbExclamation)
  Case EM_NO_VALID_LINKS:       Call MsgBox("     !", vbExclamation)
  Case EM_FILENAME_EMPTY:       Call MsgBox("     !", vbExclamation)
  Case EM_FILENAME_TOO_LONG:    Call MsgBox("        255 !", vbExclamation)
  Case EM_HAS_NO_LINKS:         Call MsgBox("      .       ", vbExclamation)
  Case EM_ID_INVALID_FORMAT:    Call MsgBox("  ", vbExclamation)
  Case EM_BOOKMARK_NOT_AVAILABLE:
    Call MsgBox("        !" & vbNewLine & _
                "   ", vbExclamation)

  ' =================  ======================
  Case IM_SCAN_MODE:            Call MsgBox("    ,   ", vbInformation)
  Case IM_IGNORE_MODE:          Call MsgBox(" ,   ", vbInformation)
  Case IM_LINKS_TODO:           Call MsgBox(Fmt(" ,     : {1}" & vbNewLine & " : {2}", unwrapped), vbInformation)
  Case IM_LINKS_DONE:           Call MsgBox(Fmt("   " & vbNewLine & " : {1}", unwrapped), vbInformation)
  Case IM_DOC_OUTER:            Call MsgBox("      .     ", vbInformation)
  Case IM_LINKS_ADDED:          Call MsgBox(Fmt("   ." & vbNewLine & "   : {1}", unwrapped), vbInformation)
  Case IM_DOC_IGNORED:          Call MsgBox(Fmt("   : {1}", unwrapped), vbInformation)
  Case IM_DOC_LINKS_DONE:       Call MsgBox("   " & vbNewLine & "  ( HT03_UpdateMarkup)!", vbInformation)
  Case IM_HYPERLINKS_DONE:      Call MsgBox(" ", vbInformation)
  Case IM_QUERY_LINK:           Call MsgBox(Fmt("        : {1}", unwrapped), vbInformation)
  Case IM_DOC_EXPRESS_ADD:      Call MsgBox(Fmt("   : {1}", unwrapped), vbInformation)
  Case IM_REMOVE_LINK_OK:       Call MsgBox(Fmt("  : {1}", unwrapped), vbInformation)
  
  Case Else:                    Call MsgBox("  ", vbCritical)
  End Select
End Function

Public Function UIAskQuestion(theCode As MsgCode, ParamArray params() As Variant) As Boolean
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Dim answer As Long: answer = vbNo
  Select Case theCode
  Case QM_ADD_TITLELINK
    answer = MsgBox(Fmt("!      " & vbNewLine & _
          "   ID: {1}" & vbNewLine & _
          "      . ?", unwrapped), vbYesNo + vbQuestion)
  Case QM_OPEN_GARANT
    answer = MsgBox(Fmt("    , " & _
            "        " & _
            "    " & vbNewLine & _
            "ID : {1}" & vbNewLine & _
            "      ?", unwrapped), vbYesNo + vbQuestion)
  Case QM_DELETE_BEFORE_ADD
    answer = MsgBox(Fmt("   ID    !" & vbNewLine & "ID: {1}" & vbNewLine & _
                    "       ?" & vbNewLine & _
                    "!!         !!", unwrapped), vbYesNo + vbExclamation)
  Case QM_DELETE_BEFORE_ADD_DUMMY
    answer = MsgBox(Fmt("   ID    !" & vbNewLine & "ID: {1}" & vbNewLine & _
                    "       ?" & vbNewLine & vbNewLine & _
                    "      ,        ", unwrapped), vbYesNo + vbExclamation)
  Case QM_RELOAD_ON_UPDATE
    answer = MsgBox(Fmt(" {1}      " & vbNewLine & _
                    "           ?", unwrapped), vbYesNo + vbExclamation)
  Case QM_RELOAD_CONFIRM:   answer = MsgBox(Fmt("     . ?", unwrapped), vbYesNo + vbExclamation)
  
  Case Else
    Call MsgBox("  ", vbCritical)
  End Select
  UIAskQuestion = answer = vbYes
End Function
