Attribute VB_Name = "z_UIMessages"
'       
Option Explicit

Public Enum MsgCode
  MSG_OK = 0
  
  EM_XLOUT_CREATE_FAIL
  EM_NAME_TAKEN
  EM_RENAME_UNLOADED
  EM_LINKBASE_FAILED
  EM_MISSING_ID
  EM_DUMMY_HAS_FILE
  EM_RENAME_FAILED
  EM_ID_INVALID_FORMAT
  EM_MERGE_VALIDATION_FAILED
  
  IM_XLOUT_OK
  IM_STATS_OK
  IM_NEXTDOC_OK
  IM_UNSTUCK_OK
  IM_RECREATE_OK
  IM_DELETE_LAW_OK
  IM_DELETE_LINKS_OK
  IM_HYPERLINKS_OK
  IM_RENAME_COMPLETE
  IM_DUMMY_OK
  IM_MERGE_OK
  
  QM_DELETE_LAW
  QM_DELETE_SELF_CONFIRM
  QM_DELETE_LINKS
  QM_MERGE_WARNING
End Enum

Private g_UI As API_UserInteraction

Public Function UserInteraction() As API_UserInteraction
  If g_UI Is Nothing Then _
    Set g_UI = New API_UserInteraction
  Set UserInteraction = g_UI
End Function

Public Function SetUserInteraction(newUI As API_UserInteraction)
  Set g_UI = newUI
End Function

Public Function UIShowMessage(theCode As MsgCode, ParamArray params() As Variant)
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Select Case theCode
  Case EM_XLOUT_CREATE_FAIL:    Call MsgBox("     ", vbExclamation)
  Case EM_NAME_TAKEN:           Call MsgBox(Fmt("      {1}", unwrapped), vbExclamation)
  Case EM_RENAME_UNLOADED:      Call MsgBox("      ", vbExclamation)
  Case EM_LINKBASE_FAILED:      Call MsgBox(Fmt("   :{1}", params), vbExclamation)
  Case EM_MISSING_ID:           Call MsgBox("  ", vbExclamation)
  Case EM_DUMMY_HAS_FILE:       Call MsgBox("    .         ", vbExclamation)
  Case EM_RENAME_FAILED:        Call MsgBox(Fmt("    {1}  {2}", unwrapped), vbExclamation)
  Case EM_ID_INVALID_FORMAT:    Call MsgBox("  ", vbExclamation)
  Case EM_MERGE_VALIDATION_FAILED: Call MsgBox("   ID   ", vbExclamation)
  
  Case IM_XLOUT_OK:             Call MsgBox(" ", vbInformation)
  Case IM_STATS_OK:             Call MsgBox(" ", vbInformation)
  Case IM_NEXTDOC_OK:           Call MsgBox("   ", vbInformation)
  Case IM_UNSTUCK_OK:           Call MsgBox("  ", vbInformation)
  Case IM_RECREATE_OK:          Call MsgBox("   ", vbInformation)
  Case IM_DELETE_LAW_OK:        Call MsgBox(Fmt(",     {1}   ", unwrapped), vbInformation)
  Case IM_DELETE_LINKS_OK:      Call MsgBox(Fmt("   {1}   .   : {2}", unwrapped), vbInformation)
  Case IM_HYPERLINKS_OK:        Call MsgBox("  ", vbInformation)
  Case IM_RENAME_COMPLETE:      Call MsgBox(Fmt(" {1} : {2}", unwrapped), vbInformation)
  Case IM_DELETE_LINKS_OK:      Call MsgBox(Fmt(" {1} ", unwrapped), vbInformation)
  Case IM_MERGE_OK:             Call MsgBox(Fmt("  {1} -> {2}", unwrapped), vbInformation)
  Case IM_DUMMY_OK:             Call MsgBox("-  " & vbNewLine & _
                                            "!       ,     ", vbInformation)

  Case Else:                    Call MsgBox("  ", vbCritical)
  End Select
End Function

Public Function UIAskQuestion(theCode As MsgCode, ParamArray params() As Variant) As Boolean
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Dim answer&: answer = vbNo
  Select Case theCode
  Case QM_DELETE_LAW:           answer = MsgBox(Fmt("!    {1}  !" & vbNewLine & "?", unwrapped), vbYesNo + vbQuestion)
  Case QM_DELETE_SELF_CONFIRM:  answer = MsgBox("    ?", vbYesNo + vbQuestion)
  Case QM_DELETE_LINKS:         answer = MsgBox(Fmt("!    {1}    !" & vbNewLine & "?", unwrapped), vbYesNo + vbQuestion)
  Case QM_MERGE_WARNING:        answer = MsgBox("!!                 . " & _
                                                "            . " & _
                                                "   Word   " & vbNewLine & _
                                                "?", vbYesNo + vbExclamation)
  
  Case Else:                    Call MsgBox("  ", vbCritical)
  End Select
  UIAskQuestion = answer = vbYes
End Function
